/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <ui-gadget.h>
#include <app_debug.h>
#include <viewmgr.h>
#include <inputmgr.h>
#include <app_define.h>

#include "defs.h"
#include "view.h"

struct _priv {
	Evas_Object *win;

	ui_gadget_h ug;

	char *pincode;
};

static void _send_message(struct _priv *priv, const char *result)
{
	service_h service;
	int r;

	r = service_create(&service);
	if (r != SERVICE_ERROR_NONE) {
		_ERR("failed to create service");
		return;
	}

	r = service_add_extra_data(service, PARAM_PIN, result);
	if (r != SERVICE_ERROR_NONE) {
		_ERR("failed to add data");
		service_destroy(service);
		return;
	}

	ug_send_message(priv->ug, service);

	service_destroy(service);
}

static void _check_pincode(struct _priv *priv, char *pincode)
{
	if (!strcmp(pincode, priv->pincode)) {
		viewmgr_hide_view(VIEW_PIN);
		viewmgr_update_view(VIEW_USER_EDIT, UPDATE_TYPE_PIN_RESULT,
				pincode);
	} else {
		_send_message(priv, MESSAGE_FAIL);
	}
}

static void _ug_result_cb(ui_gadget_h ug, service_h result, void *p)
{
	char *pincode;
	int r;

	if (!p) {
		_ERR("Invalid argument");
		return;
	}

	r = service_get_extra_data(result, PARAM_PIN, &pincode);
	if (r != SERVICE_ERROR_NONE || !pincode) {
		_ERR("failed to get data");
		return;
	}

	_check_pincode(p, pincode);

	free(pincode);
}

static void _ug_destroy_cb(ui_gadget_h ug, void *p)
{
	viewmgr_hide_view(VIEW_PIN);
}

static ui_gadget_h _load_ug(struct _priv *priv)
{
	ui_gadget_h ug;
	struct ug_cbs cb = {0,};

	cb.result_cb = _ug_result_cb;
	cb.destroy_cb = _ug_destroy_cb;
	cb.priv = priv;

	ug = ug_create(NULL, UG_ID_PIN, UG_MODE_FRAMEVIEW, NULL, &cb);

	return ug;
}

static Evas_Object *_create(Evas_Object *win, void *data)
{
	struct _priv *priv;

	if (!win) {
		_ERR("Invalid argument");
		return NULL;
	}

	priv = calloc(1, sizeof(*priv));
	if (!priv) {
		_ERR("failed to calloc priv");
		return NULL;
	}

	priv->win = win;

	viewmgr_set_view_data(VIEW_PIN, priv);

	return win;
}

static void _show(void *data)
{
	struct _priv *priv;
	ui_gadget_h ug;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;

	ug = _load_ug(priv);
	if (!ug) {
		_ERR("failed to create ug");
		return;
	}

	priv->ug = ug;
}

static void _hide(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;

	if (priv->ug)
		ug_destroy(priv->ug);

	free(priv->pincode);
	priv->pincode = NULL;
}

static void _destroy(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;

	free(priv);
}

static void _update(void *view_data, int update_type, void *data)
{
	struct _priv *priv;

	if (!view_data) {
		_ERR("Invalid argument");
		return;
	}

	priv = view_data;

	switch (update_type) {
	case UPDATE_TYPE_PIN_CODE:
		if (!data)
			return;

		priv->pincode = strdup(data);
		break;
	default:
		_ERR("Invalid type");
		return;
	}
}

static view_class vclass = {
	.view_id = VIEW_PIN,
	.create = _create,
	.show = _show,
	.hide = _hide,
	.destroy = _destroy,
	.update = _update
};

view_class *view_pin_get_vclass(void)
{
	return &vclass;
}
